from binance.client import Client
import math
import logging
from web_app.models import *
from django.utils import timezone

logger = logging.getLogger(__name__)
class Binance_client:
    def __init__(self, BINANCE_API_KEY, BINANCE_SECRET_KEY, is_usa=False):
        if(is_usa):
            self.client = Client(BINANCE_API_KEY, BINANCE_SECRET_KEY, tld='us')
        else:
            self.client = Client(BINANCE_API_KEY, BINANCE_SECRET_KEY)

    def get_balance(self, symbol):
        account_info = self.client.get_account()
        for balance in account_info['balances']:
            if balance['asset'] == symbol:
                current_balance = float(balance['free'])
                return current_balance
        else:
            return 0

    def precise_quantity_binance(self, symbol, quantity):
        info = self.client.get_symbol_info(symbol)
        for x in info["filters"]:
            if x["filterType"] == "LOT_SIZE":
                stepSize = float(x["stepSize"])

        truncate_num = math.log10(1 / stepSize)
        quantity = math.floor((quantity) * 10 ** truncate_num) / 10 ** truncate_num
        return quantity
    def coin_buy(self, setting, symbol):
        try:
            username = setting.username
            bought_order = Order.objects.filter(status="bought", symbol=symbol, username=username).order_by('-buy_time').last()
            if(bought_order != None):
                logger.info("Currently there is bought order for this coin. Skipping.")
                return False
            coin_price = float(self.client.get_ticker(symbol=symbol)['lastPrice'])
            order_size = setting.order_size
            order_qty = self.precise_quantity_binance(symbol, order_size / coin_price)
            order_res = self.client.create_order(
                symbol=symbol,
                side="BUY",
                type="MARKET",
                quantity=order_qty
            )
            logger.info("Buy order created successfully")
            logger.info(str(order_res))
            order = Order()
            order.username = username
            order.symbol = symbol
            order.status = "bought"
            order.buy_time = timezone.now()
            order.buy_qty = float(order_res["executedQty"])
            order.buy_price = float(order_res["fills"][0]["price"])
            order.buy_value = order.buy_qty * order.buy_price
            order.buy_fees_value = sum(float(item["commission"]) for item in order_res["fills"]) * order.buy_price
            order.save()
            logger.info("Buy order recorded to db successfully")
        except Exception as e:
            logger.error("Error in coin_buy: " + str(e))
            return False

    def coin_sell(self, setting, symbol):
        try:
            username = setting.username
            bought_order = Order.objects.filter(status="bought", symbol=symbol, username=username).order_by('-buy_time').last()
            if(bought_order == None):
                logger.info("Currently there is no bought order for this coin. Skipping.")
                return False
            order_qty = bought_order.buy_qty
            current_symbol_qty = self.get_balance(symbol.replace("USDT", "").replace("USDC", ""))
            if(current_symbol_qty < order_qty):
                order_qty = current_symbol_qty
                logger.info("current_symbol_qty is taking as qty: " + str(current_symbol_qty))
            order_res = self.client.create_order(
                symbol=symbol,
                side="SELL",
                type="MARKET",
                quantity=order_qty
            )
            logger.info("Sell order created successfully")
            logger.info(str(order_res))
            bought_order.sell_time = timezone.now()
            bought_order.status = "sold"
            bought_order.sell_qty = float(order_res["executedQty"])
            bought_order.sell_price = float(order_res["fills"][0]["price"])
            bought_order.sell_value = bought_order.sell_qty * bought_order.sell_price
            bought_order.sell_fees_value = sum(float(item["commission"]) for item in order_res["fills"])
            bought_order.profit = bought_order.sell_value - bought_order.buy_value - bought_order.buy_fees_value - bought_order.sell_fees_value
            bought_order.profit_perc = ( bought_order.profit / bought_order.buy_value ) * 100
            bought_order.save()
            logger.info("Buy order recorded to db successfully")
        except Exception as e:
            logger.error("Error in coin_buy: " + str(e))
            return False