from django.shortcuts import render, redirect
from django.contrib.auth import authenticate, login, logout
from django.contrib.auth.forms import AuthenticationForm
from django.http import HttpResponse
from django.views.decorators.csrf import csrf_exempt
import logging
from web_app.models import *
from .forms import SettingEditForm
from .utils.binance_client import Binance_client
from django.http import JsonResponse

logger = logging.getLogger(__name__)
@csrf_exempt
def tv_webhook(request, username):
    if request.method == 'POST':
        alert = request.body.decode('utf-8').upper()
        logger.info("#################### ALERT from tradingview: " + alert)
        logger.info("username: " + username)
        side = alert.split(" ")[0]
        symbol = alert.split(" ")[1]
        setting = Setting.objects.filter(username=username).get()
        client = Binance_client(setting.binance_api_key, setting.binance_secret, setting.is_usa)
        if(side == "BUY"):
            client.coin_buy(setting, symbol)
        elif(side == "SELL"):
            client.coin_sell(setting, symbol)
        return HttpResponse("Done")

def login_view(request):
    if request.method == 'POST':
        form = AuthenticationForm(request, data=request.POST)
        if form.is_valid():
            username = form.cleaned_data.get('username')
            password = form.cleaned_data.get('password')
            user = authenticate(request, username=username, password=password)
            if user is not None:
                login(request, user)
                return redirect('home')
    else:
        form = AuthenticationForm()
    return render(request, 'web_app/login.html', {'form': form})

def pre_process_order(value, decimal_places):
    if(value == None):
        return ""
    else:
        try:
            return round(value, decimal_places)
        except:
            return  value
def pre_process_orders(orders):
    for order in orders:
        order.buy_value = pre_process_order(order.buy_value, 3)
        order.buy_qty = pre_process_order(order.buy_qty, 6)
        order.buy_price = pre_process_order(order.buy_price, 8)  # Change 2 to the desired decimal points
        order.buy_fees_value = pre_process_order(order.buy_fees_value, 4)
        order.sell_value = pre_process_order(order.sell_value, 3)
        order.sell_qty = pre_process_order(order.sell_qty, 6)
        order.sell_price = pre_process_order(order.sell_price, 8)  # Change 2 to the desired decimal points
        order.sell_fees_value = pre_process_order(order.sell_fees_value, 4)
        order.profit = pre_process_order(order.profit, 3)
        order.profit_perc = pre_process_order(order.profit_perc, 2)
        order.sell_time = pre_process_order(order.sell_time, 0)
    return orders
def home_view(request):
    if not request.user.is_authenticated:
        return redirect('login')
    username = request.user.username
    orders = Order.objects.filter(username=username).order_by('buy_time')
    return render(request, 'web_app/home.html', {'username': username, 'orders': pre_process_orders(orders)})

def logout_view(request):
    logout(request)
    return redirect('login')

def edit_setting(request, username):
    setting = Setting.objects.get(username=username)
    if request.method == 'POST':
        form = SettingEditForm(request.POST, instance=setting)
        if form.is_valid():
            form.save()
            return redirect('home')
    else:
        form = SettingEditForm(instance=setting)

    return render(request, 'web_app/edit_setting.html', {'form': form})
@csrf_exempt
def health(request):
    return JsonResponse({"status": "ok"})